# This file was auto-generated by Fern from our API Definition.

import typing

AlertTriggerConfigType = typing.Union[
    typing.Literal[
        "scope:project", "threshold:feedback_score", "threshold:cost", "threshold:latency", "threshold:errors"
    ],
    typing.Any,
]
