# This file was auto-generated by Fern from our API Definition.

import typing

AlertTriggerEventType = typing.Union[
    typing.Literal[
        "trace:errors",
        "trace:feedback_score",
        "trace_thread:feedback_score",
        "prompt:created",
        "prompt:committed",
        "trace:guardrails_triggered",
        "prompt:deleted",
        "experiment:finished",
        "trace:cost",
        "trace:latency",
    ],
    typing.Any,
]
